<?php
namespace Teknisa\FAI\Modules\Parametros\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DataSource\FilterCriteria;
use \Zeedhi\Framework\DataSource\Manager;
use \Zeedhi\Framework\DataSource\ParameterBag;

class Fai05100GrupoClientesParaFaturamento extends \Teknisa\GER\Controllers\TeknisaCrud {
  protected $crudService;
  protected $parameterBag;
  protected $entityManager;
  protected $environment;
  protected $dataSourceManager;
  protected $util;
  protected $newCode;
  protected $connection;
  protected $novoCodigo;
  
  public function __construct($crudService, ParameterBag $parameterBag, $environment, $entityManager, $dataSourceManager, $util, $newCode, $connection, $novoCodigo) {
    parent::__construct($crudService, $parameterBag);
    $this->crudService = $crudService;
    $this->entityManager = $entityManager;
    $this->environment = $environment;
    $this->dataSourceManager = $dataSourceManager;
    $this->util = $util;
    $this->newCode = $newCode;
    $this->connection = $connection;    
    $this->novoCodigo = $novoCodigo;
  }
  
  public function salvarGrupo(DTO\Request $request, DTO\Response $response) {
    try {
      $this->connection->beginTransaction();
      
      $this->dataSourceName = 'grupcliefat';
      $this->parameterBagColumns = array();
			
			$rows = $this->crudService->getRowsToSave($request, $this->dataSourceName);
			
			foreach ($rows as $row) {
			  if(!$row["__is_new"]) {
  			  $row["DTINCLUSAO"] = substr($row["DTINCLUSAO"], 0, 10);
  			  $row["DTULTATU"] = substr($row["DTULTATU"], 0, 10);
			  } else {
			    $row["NRGRUPCLIEFAT"] = intval($this->novoCodigo->getCodeByTableName('GRUPCLIEFAT', $row['NRORG']));
			  }
			  $row["VRMINFAT"] = round($row["VRMINFAT"], 2);
				$this->crudService->save($this->dataSourceName, array($row));
				
				$resp[] = $row["NRGRUPCLIEFAT"];
				$dataSet = new Dataset('nrGrupClieFat', $resp);
        $response->addDataSet($dataSet, 'nrGrupClieFat');
			}			

      $this->connection->commit();
    } catch(\Exception $e) {
			$response->addDataSet(new DataSet("errorInfo", array("erroSalvarGrupo" => $e->getMessage())));
			$response->setError(new Error("erroSalvarGrupo", 500));      
    }
  }
  
  public function verificaClienteExiste($row) {
    $query = "
    SELECT MAX(G.NRGRUPCLIEFAT || ' - ' || G.NMGRUPCLIEFAT) AS GRUPO
      FROM ITGRUPCLIEFAT IC
           LEFT JOIN GRUPCLIEFAT G ON G.NRGRUPCLIEFAT = IC.NRGRUPCLIEFAT
     WHERE IC.CDFILIAL = :P_CDFILIAL
       AND IC.CDCLIENTE = :P_CDCLIENTE
       AND IC.NRGRUPCLIEFAT <> :P_NRGRUPCLIEFAT";
    $params = array(
      "P_CDFILIAL" => $row["CDFILIAL"],
      "P_CDCLIENTE" => $row["CDCLIENTE"],
      "P_NRGRUPCLIEFAT" => $row["NRGRUPCLIEFAT"]
    );
    
    $duplicatedClients = $this->entityManager->getConnection()->fetchAll($query, $params);
    return $duplicatedClients;
  }
  
  public function verificaClientePrincipal($row) {
    $queryUpdateLimpaIdPrincipal = "UPDATE ITGRUPCLIEFAT SET IDPRINCIPAL = 'N' WHERE CDFILIAL = :P_CDFILIAL AND NRGRUPCLIEFAT = :P_NRGRUPCLIEFAT AND CDCLIENTE <> :P_CDCLIENTE";
    
    $paramsUpdate = array(
      "P_CDFILIAL"      => $row["CDFILIAL"],
      "P_NRGRUPCLIEFAT" => $row["NRGRUPCLIEFAT"],
      "P_CDCLIENTE"     => $row["CDCLIENTE"]
    );
    if($row["IDPRINCIPAL"] == 'S') {
      $stmt = $this->connection->prepare($queryUpdateLimpaIdPrincipal);
      $stmt->execute($paramsUpdate);  
    }
  }
  
  public function salvarCliente(DTO\Request $request, DTO\Response $response) {
    try {
      $this->connection->beginTransaction();
      $this->dataSourceName = 'itgrupcliefat';
      $this->parameterBagColumns = array();
      
      $rows = $this->crudService->getRowsToSave($request, $this->dataSourceName);
      foreach($rows as $row) {
			  if(!$row["__is_new"]) {
  			  $row["DTINCLUSAO"] = substr($row["DTINCLUSAO"], 0, 10);
  			  $row["DTULTATU"] = substr($row["DTULTATU"], 0, 10);
  			  $this->verificaClientePrincipal($row);
			  } else {
			    $row["NRITGRUPCLIEFAT"] = intval($this->novoCodigo->getCodeByTableName('ITGRUPCLIEFAT', $row['NRORG']));
			    $verifyDuplicatedClients = $this->verificaClienteExiste($row);
			    if(!empty($verifyDuplicatedClients)) {
			      if(!empty($verifyDuplicatedClients[0]['GRUPO'])){
              $response->addDataSet(new DataSet ("DuplicatedClientsValues", $verifyDuplicatedClients[0]));
              return $response->setError(new Error("erroSalvarCliente", 500));
			      }
			    }
			    $this->verificaClientePrincipal($row);
			  }  
				$this->crudService->save($this->dataSourceName, array($row));
				$this->atualizaItemNota($row['NRGRUPCLIEFAT'], $row['CDCLIENTESERV'], $row['CDFILIALSERV'], $row['CDSERVFILI']);
				$this->connection->commit();
				
				if($row["IDPRINCIPAL"] == 'S') {
				  $response->addDataSet(new DataSet("CDCLIENTEPRIN", array($row["CDCLIENTE"])));
				}
				$response->addDataSet(new DataSet("NRITGRUPCLIEFAT", array($row["NRITGRUPCLIEFAT"])));
      }
    } catch(\Exception $e) {
			$response->addDataSet(new DataSet("errorInfo", array("erroSalvarCliente" => $e->getMessage())));
			$response->setError(new Error("erroSalvarCliente", 500));      
			$this->connection->rollback();
    }
  }
  
  private function atualizaItemNota($nrGrupClieFat, $cdCliente, $cdFilial, $cdServFili){
    if($cdCliente != 'NAO_ALTERAR') {
      $query = "
        UPDATE GRUPCLIEFAT
           SET CDCLIENTE = :P_CDCLIENTE,
               CDFILIALSERV = :P_CDFILIAL,
               CDSERVFILI = :P_CDSERVFILI,
               DTULTATU = SYSDATE,
               CDOPERULTATU = :P_CDOPERADOR
         WHERE NRGRUPCLIEFAT = :P_NRGRUPCLIEFAT";
      $params = array(
        "P_CDFILIAL" => $cdFilial,
        "P_CDCLIENTE" => $cdCliente,
        "P_CDSERVFILI" => $cdServFili,
        "P_CDOPERADOR" => $this->environment->getCdOperador(),
        "P_NRGRUPCLIEFAT" => $nrGrupClieFat
      );
      $sql = $this->entityManager->getConnection()->prepare($query);
		  $sql->execute($params);
    }
  }
  
  public function salvarServico(DTO\Request $request, DTO\Response $response) {
    try {
      $this->connection->beginTransaction();
      $this->dataSourceName = 'servgrupcliefat';
      $this->parameterBagColumns = array();
      $rows = $this->crudService->getRowsToSave($request, $this->dataSourceName);
      foreach($rows as $row) {
			  if($row["__is_new"]) {
			    $row["NRSERVGRUPCLIEFAT"] = intval($this->novoCodigo->getCodeByTableName('SERVGRUPCLIEFAT', $row['NRORG']));
			  }
			  $this->crudService->save($this->dataSourceName, array($row));
			}
			$response->addDataSet(new DataSet("NRSERVGRUPCLIEFAT", array($row["NRSERVGRUPCLIEFAT"])));
			$this->connection->commit();
    } catch(\Exception $e) {
			$response->addDataSet(new DataSet("errorInfo", array("erroSalvarServico" => $e->getMessage())));
			$response->setError(new Error("erroSalvarServico", 500));
			$this->connection->rollback();
    }
  }
  
  public function salvarServicoInline(DTO\Request $request, DTO\Response $response){
    $this->salvarServico($request, $response);
  }
  
  public function exclusaoClientePrincipal(DTO\Request $request, DTO\Response $response) {
    try {
      $this->connection->beginTransaction();
      $row = $request->getRow();
      $this->atualizaItemNota($row['NRGRUPCLIEFAT'], null, null, null);
      $this->connection->commit();
    } catch(\Exception $e) {
      $this->connection->rollback();
    }
  }
}